//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.1
// Board : Seeed nRF52 Borads / Seeed XIAO nRF52840 Sense 
//----------------------------------------------------------------------------------------------

#include <Arduino.h>
#include <Adafruit_TinyUSB.h>   // For Serial.print()
#include <bluefruit.h>          // /Arduino15/packages/Seeeduino/hardware/nrf52/1.1.1/libraries/Bluefruit52Lib/src
#include <Adafruit_SPIFlash.h>  // Need to be deleted /Documents/Arduino/libraries/SdFat

// on board Flash P25Q16H
SPIFlash_Device_t const P25Q16H {
  .total_size = (1UL << 21), // 2MiB
  .start_up_time_us = 10000, // Don't know where to find that value  
  .manufacturer_id = 0x85,
  .memory_type = 0x60,
  .capacity = 0x15,
  .max_clock_speed_mhz = 55,
  .quad_enable_bit_mask = 0x02, // Datasheet p. 27
  .has_sector_protection = 1,   // Datasheet p. 27
  .supports_fast_read = 1,      // Datasheet p. 29
  .supports_qspi = 1,           // Obviously
  .supports_qspi_writes = 1,    // Datasheet p. 41
  .write_status_register_split = 1, // Datasheet p. 28
  .single_status_byte = 0,      // 2 bytes
  .is_fram = 0,                 // Flash Memory
};
Adafruit_FlashTransport_QSPI flashTransport;
Adafruit_SPIFlash flash(&flashTransport);

//*********************************************************************************************
void setup()
{
  // Serial port initialization
  Serial.begin(115200);
//  while (!Serial);
  delay(2000);
    
  // Pin initialization
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);

  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_GREEN, HIGH);

  // Enable DC-DC converter
  NRF_POWER->DCDCEN = 1;            // Enable DC/DC converter for REG1 stage

  delay(300);  
      

  // ************************************************************************************
  #define PowerDownTime 100      // time of DeepPowerDown [mS]
  #define WakeUpTime 1000          // time of Wake up [mS]
  
  Serial.println("flashTransport.runCommand()");
  Serial.println("enter");
  flashTransport.begin();
  flashTransport.runCommand(0xB9);  // enter deep power-down mode
  delayMicroseconds(5);             // tDP=3uS
  flashTransport.end();    
  delay(PowerDownTime);

  Serial.println("release");
  flashTransport.begin();   
  flashTransport.runCommand(0xAB);  // 0xAB release Deep Power-down                                    
  delayMicroseconds(10);            // tRES=8uS
  flashTransport.end();    
  delay(WakeUpTime);

  // ************************************************************************************
  // Initialize flash library and check its chip ID.
  Serial.println("Initialized Flash");
  if (!flash.begin(&P25Q16H, 1)) {    // onBoard Flash    
    Serial.println("Error, failed to initialize flash chip!");
    while(1) {
      digitalWrite(LED_RED, LOW);
      delay(10);
      digitalWrite(LED_RED, HIGH);
      delay(100);
    }
  }
  Serial.print("Flash chip JEDEC ID: 0x"); Serial.println(flash.getJEDECID(), HEX);
  delay(WakeUpTime);
  
  Serial.println("End of initialization");
    while(1) {
      digitalWrite(LED_GREEN, LOW);
      delay(10);
      digitalWrite(LED_GREEN, HIGH);
      delay(100);
    }
}

void loop()
{      

}
